#include <stdio.h>
#include <math.h>

/*-------------------------------------------------------------------*/

void rescale (glob, sizex, sizey, scale_factor)
float  		glob [];
unsigned int	sizex, sizey;
float		scale_factor;
{
long		i;

/* renormalize */
for (i=0; i<sizex*sizey; i++) {
   glob [i] *= scale_factor ;
   glob [i] -= (int) glob[i];
   }
}

/*-------------------------------------------------------------------*/

void expand (glob, sizex, sizey, scale_factor)
float  		glob [];
unsigned int	sizex, sizey;
float		scale_factor;
{
long		i;

/* renormalize */
for (i=0; i<sizex*sizey; i++) {
   glob [i] *= scale_factor ;
   }
}

/*-------------------------------------------------------------------*/

void takelog (glob, sizex, sizey, scale_factor)
float  		glob [];
unsigned int	sizex, sizey;
float		scale_factor;
{
long		i;
float		ska;

ska = exp (scale_factor) - 1.0;
/* renormalize */
for (i=0; i<sizex*sizey; i++) {
   glob [i] = (float) log ((double) (ska * glob[i] + 1.0)) / scale_factor;
   }
}

/*-------------------------------------------------------------------*/

void edge (row_m1, row, row_p1, row_out, sizex, scale_factor)
float  		row_m1 []; /* row minus 1 */
float  		row [];    /* row */
float  		row_p1 []; /* row plus 1 */
float  		row_out [];
unsigned int	sizex;
float		scale_factor;
{
long		i;
float 		oops;

oops = 1.0 / scale_factor;
/* first pixel is special */
row_out[0] = 3.0*row[0] -row[1] -row_p1[0] -row_m1[0];
row_out[0] *= oops;

for (i=1; i<sizex-1; i++) {
   row_out[i] = 4.0*row[i] -row[i+1] -row[i-1] -row_p1[i] -row_m1[i];
   row_out[i] *= oops;
   }

/* last pixel is special */
row_out[sizex-1] = 3.0*row[sizex-1] -row[sizex-2] -row_p1[sizex-1] -row_m1[sizex-1];
row_out[sizex-1] *= oops;
}

/*-------------------------------------------------------------------*/

void dredge (row_m1, row, row_p1, row_out, sizex, scale_factor)
float  		row_m1 []; /* row minus 1 */
float  		row [];    /* row */
float  		row_p1 []; /* row plus 1 */
float  		row_out [];
unsigned int	sizex;
float		scale_factor;
{
long		i;

/* first pixel is special */
if ((fabs (row[0]-row[1]) > scale_factor)  ||
    (fabs (row_p1[0]-row[0]) > scale_factor)  ||
    (fabs (row_m1[0]-row[0]) > scale_factor)) {
    row_out [0] = 0.8;
    } else {
    row_out[0] = 0.0;
    }

for (i=1; i<sizex-1; i++) {
   if ((fabs (row[i]-row[i-1]) > scale_factor)  ||
       (fabs (row[i+1]-row[i]) > scale_factor)  ||
       (fabs (row_p1[i]-row[i]) > scale_factor)  ||
       (fabs (row_m1[i]-row[i]) > scale_factor)) {
        row_out[i] = 0.8;
        } else {
        row_out[i] = 0.0;
        }
   }

/* last pixel is special */
if ((fabs (row[sizex-1]-row[sizex-2]) > scale_factor)  ||
    (fabs (row_p1[sizex-1]-row[sizex-1]) > scale_factor)  ||
    (fabs (row_m1[sizex-1]-row[sizex-1]) > scale_factor)) {
    row_out[sizex-1] = 0.8;
    } else {
    row_out[sizex-1] = 0.0;
    }
}

/*-------------------------------------------------------------------*/

void contour (row_m1, row, row_p1, row_out, sizex)
float  		row_m1 []; /* row minus 1 */
float  		row [];    /* row */
float  		row_p1 []; /* row plus 1 */
float  		row_out [];
unsigned int	sizex;
{
long		i;

/* first pixel is special */
if ( (((int) row[0]) != ((int) row[1]))  ||
     (((int) row[0]) != ((int) row_p1[0]))  ||
     (((int) row[0]) != ((int) row_m1[0])) ) {
     row_out[0] = 0.8;
     } else {
     row_out[0] = 0.1;
     }

for (i=1; i<sizex-1; i++) {
   if ( (((int) row[i]) != ((int) row[i-1]))  ||
        (((int) row[i]) != ((int) row[i+1]))  ||
        (((int) row[i]) != ((int) row_p1[i]))  ||
        (((int) row[i]) != ((int) row_m1[i])) ) {
        row_out[i] = 0.8;
        } else {
        row_out[i] = 0.1;
        }
   }

/* last pixel is special */
if ( (((int) row[sizex-1]) != ((int) row[sizex-2]))  ||
     (((int) row[sizex-1]) != ((int) row_p1[sizex-1]))  ||
     (((int) row[sizex-1]) != ((int) row_m1[sizex-1])) ) {
     row_out[sizex-1] = 0.8;
     } else {
     row_out[sizex-1] = 0.1;
     }

}

/*-------------------------------------------------------------------*/

#ifdef CONTOUR_STUFF
#define SCALE 10.0
#define THRESH 0.9

#define PIX_WID 1600
#define PIX_HIG 1280
extern FILE *Fopen();
extern FILE *Fopenr();

void main (argc, argv) 
int argc;
char *argv[];

{
static float	data_in [PIX_WID*4];	/* my data array */
static float	data_out [PIX_WID*4];	/* my data array */
float		*ptr_a; 		/* pointer into data array */
float		*ptr_b; 		/* pointer into data array */
float		*ptr_c; 		/* pointer into data array */
float		*tmp; 		/* pointer into data array */
unsigned int	data_width, data_height;/* data array dimensions */
int		i,j;
char 		str[80];
FILE		*fp_in, *fp_out;
float 		scale_fact;

if ( argc < 3) {
   printf ("usage: dx <input file> <output file> \n");
   return;
   }

/*-----------------------------------------------*/
/* open input file */
if ( (fp_in = Fopenr (argv[1], ".flo")) == NULL) {
   printf (" Can't open input file \n");
   return;
   }
if ( NULL == fgets (str, 80, fp_in)) {
   printf (" Can't read input file \n");
   fclose (fp_in);
   return;
   }

sscanf (str, "%d %d", &data_width, &data_height);
printf (" input file has width %d height %d \n", data_width, data_height);

/*-----------------------------------------------*/
/* open output file */
if ( (fp_out = Fopen (argv[2], ".flo")) == NULL) {
   printf (" Can't open output file \n");
   return;
   }

/* dump the size to output */
fprintf (fp_out, "%d %d\n", data_width, data_height);

/*-----------------------------------------------*/
ptr_a = data_in;
ptr_b = ptr_a + data_width;
ptr_c = ptr_b + data_width;
/* read in first two rows */
fread(ptr_a, sizeof(float), data_width, fp_in);
fread(ptr_b, sizeof(float), data_width, fp_in);

/* first row is special */
/*
(void) rescale (ptr_a, data_width, 1, SCALE);
(void) rescale (ptr_b, data_width, 1, SCALE);
(void) dredge (ptr_a, ptr_b, ptr_b, data_out, data_width, THRESH);
*/
(void) expand (ptr_a, data_width, 1, SCALE);
(void) expand (ptr_b, data_width, 1, SCALE);
(void) contour (ptr_a, ptr_b, ptr_b, data_out, data_width);
fwrite (data_out, sizeof(float), data_width, fp_out);

for (i=1; i<data_height-1; i++) {
    printf (" working on row %d\n", i);

    /* read floating point data */
    fread(ptr_c, sizeof(float), data_width, fp_in);

/*
    (void) rescale (ptr_c, data_width, 1, SCALE);
    (void) dredge (ptr_a, ptr_b, ptr_c, data_out, data_width, THRESH);
*/
    (void) expand (ptr_c, data_width, 1, SCALE);
    (void) contour (ptr_a, ptr_b, ptr_c, data_out, data_width);

    /* dump the floating point data */
    fwrite (data_out, sizeof(float), data_width, fp_out);

    /* do the Harlem shuffle */
    tmp = ptr_a;
    ptr_a = ptr_b;
    ptr_b = ptr_c;
    ptr_c = tmp;
    }

/* last row is special */
/*
(void) dredge (ptr_b, ptr_b, ptr_c, data_out, data_width, THRESH);
*/
(void) contour (ptr_b, ptr_b, ptr_c, data_out, data_width);
fwrite (data_out, sizeof(float), data_width, fp_out);

fclose (fp_in);
fclose (fp_out);

}
#endif /* CONTOUR_STUFF */

/*-------------------------------------------------------------------*/

#define SCALE 0.2

#define PIX_WID 1600
#define PIX_HIG 1280
extern FILE *Fopen();
extern FILE *Fopenr();

void main (argc, argv) 
int argc;
char *argv[];

{
static float	data_in [PIX_WID*4];	/* my data array */
static float	data_out [PIX_WID*4];	/* my data array */
float		*tmp; 		/* pointer into data array */
unsigned int	data_width, data_height;/* data array dimensions */
int		i,j;
char 		str[80];
FILE		*fp_in, *fp_out;
float 		scale_fact;

if ( argc < 3) {
   printf ("usage: dx <input file> <output file> \n");
   return;
   }

/*-----------------------------------------------*/
/* open input file */
if ( (fp_in = Fopenr (argv[1], ".flo")) == NULL) {
   printf (" Can't open input file \n");
   return;
   }
if ( NULL == fgets (str, 80, fp_in)) {
   printf (" Can't read input file \n");
   fclose (fp_in);
   return;
   }

sscanf (str, "%d %d", &data_width, &data_height);
printf (" input file has width %d height %d \n", data_width, data_height);

/*-----------------------------------------------*/
/* open output file */
if ( (fp_out = Fopen (argv[2], ".flo")) == NULL) {
   printf (" Can't open output file \n");
   return;
   }

/* dump the size to output */
fprintf (fp_out, "%d %d\n", data_width, data_height);

/*-----------------------------------------------*/

for (i=0; i<data_height; i++) {
    if (i%20 ==0) printf (" working on row %d\n", i);

    /* read floating point data */
    fread(data_in, sizeof(float), data_width, fp_in);

    (void) expand (data_in, data_width, 1, SCALE);

    /* dump the floating point data */
    fwrite (data_in, sizeof(float), data_width, fp_out);

}

fclose (fp_in);
fclose (fp_out);

}

/* --------------- END OF FILE ------------------------- */
